## @package officegenerator
## @brief Generate office files with predefined styles
from officegenerator.libodfgenerator import Color, ColumnWidthODS, ODS_Read, ODS_Write,  ODT,  OdfCell,  OdfSheet
from officegenerator.libxlsxgenerator import OpenPyXL,  ColumnWidthXLSX
from officegenerator.commons import *

